package com.taais.biz.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.common.core.config.TaaisConfig;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.core.utils.StringUtils;
import com.taais.common.core.utils.file.FileUploadUtils;
import com.taais.common.core.utils.file.FileUtils;
import com.taais.common.core.utils.file.UnPackedUtil;
import com.taais.common.excel.utils.ExcelUtil;
import com.taais.common.log.annotation.Log;
import com.taais.common.log.enums.BusinessType;
import com.taais.common.web.annotation.RepeatSubmit;
import com.taais.common.web.core.BaseController;
import com.taais.biz.domain.Data;
import com.taais.biz.domain.bo.DataBo;
import com.taais.biz.domain.vo.DataVo;
import com.taais.biz.service.IDataService;
import com.taais.system.config.ServerConfig;
import com.taais.system.domain.vo.SysOssUploadVo;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import net.lingala.zip4j.model.FileHeader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * 数据管理Controller
 *
 * @author km
 * 2024-04-22
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/demo/data")
public class DataController extends BaseController {
    private static final Logger log = LoggerFactory.getLogger(DataController.class);
    @Resource
    private IDataService dataService;
    @Autowired
    private ServerConfig serverConfig;

    /**
     * 查询数据管理列表
     */
    @SaCheckPermission("demo:data:list")
    @GetMapping("/list")
    public CommonResult<PageResult<DataVo>> list(DataBo dataBo) {

        return CommonResult.success(dataService.selectPage(dataBo));

    }

    /**
     * 导出数据管理列表
     */
    @SaCheckPermission("demo:data:export")
    @Log(title = "数据管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DataBo dataBo) {
        List<DataVo> list = dataService.selectList(dataBo);
        ExcelUtil.exportExcel(list, "数据管理", DataVo.class, response);
    }

    /**
     * 获取数据管理详细信息
     */
    @SaCheckPermission("demo:data:query")
    @GetMapping(value = "/{id}")
    public CommonResult<DataVo> getInfo(@PathVariable Long id) {

        return CommonResult.success(dataService.selectById(id));
    }

    /**
     * 新增数据管理
     */
    @SaCheckPermission("demo:data:add")
    @Log(title = "数据管理", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody DataBo dataBo) {
        boolean inserted = dataService.insert(dataBo);
        if (!inserted) {
            return CommonResult.fail("新增数据管理记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改数据管理
     */
    @SaCheckPermission("demo:data:edit")
    @Log(title = "数据管理", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody DataBo dataBo) {
        Boolean updated = dataService.update(dataBo);
        if (!updated) {
            return CommonResult.fail("修改数据管理记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除数据管理
     */
    @SaCheckPermission("demo:data:remove")
    @Log(title = "数据管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = dataService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除数据管理记录失败!");
        }
        return CommonResult.success();
    }

    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "用户数据", DataVo.class, response);
    }

    @PostMapping("/zip/upload")
    public CommonResult<SysOssUploadVo> uploadZipFile(@RequestParam("file") MultipartFile file, Data dataInfo) throws Exception {
        String originalFilename = file.getOriginalFilename();
        assert originalFilename != null;
        String suffix = StringUtils.substring(originalFilename, originalFilename.lastIndexOf("."), originalFilename.length());
        // 解压
        String basedir = TaaisConfig.getUploadPath();
        // 解压目录
        File zipOrRarTemp = FileUploadUtils.getAbsoluteFile(basedir, FileUploadUtils.extractFilename2(file));
        String destZip = zipOrRarTemp.getAbsolutePath();

        file.transferTo(Paths.get(destZip));
        String dest = zipOrRarTemp.getParent();

        //取出最大值
        Integer idMax = dataService.getIdMax();
        if (idMax == null) {
            idMax = 1;
        } else {
            idMax = idMax + 1;
        }

        List<Data> dataList = new ArrayList<>();
        if (suffix.equals(".zip")) {
            List<FileHeader> fileheaders = UnPackedUtil.unPackZip(zipOrRarTemp, dest);
            //添加解压目录
            for (FileHeader fileHeader : fileheaders) {
                boolean isOk = initFileInfo(dest, idMax, fileHeader.isDirectory(), fileHeader.getFileName(), dataList, dataInfo);
                if (isOk) {
                    idMax++;
                }
            }
        } else if (suffix.equals(".rar")) {
            List<com.github.junrar.rarfile.FileHeader> fileheaders = UnPackedUtil.unPackRar(zipOrRarTemp, dest);
            //添加解压目录
            for (com.github.junrar.rarfile.FileHeader fileHeader : fileheaders) {
                boolean isOk = initFileInfo(dest, idMax, fileHeader.isDirectory(), fileHeader.getFileName(), dataList, dataInfo);
                if (isOk) {
                    idMax++;
                }
            }
        }

        if (dataInfo.getLabeled()) {
            List<Data> unmarkedData = dataList.stream()
                    .filter(data -> Boolean.FALSE.equals(data.getLabeled()))
                    .toList();

            // 计算已标注和未标注的数据数量
            long markedCount = dataList.stream()
                    .filter(data -> Boolean.TRUE.equals(data.getLabeled()))
                    .count();
            long unmarkedCount = unmarkedData.size();

            // 如果存在未标注的数据，则返回错误信息
            if (unmarkedCount > 0) {
                String format = String.format("错误: 已标注文件 %d 个，未标注文件 %d 个", markedCount, unmarkedCount);
                return CommonResult.fail(format);
            }
        }

        AtomicBoolean isTrue = new AtomicBoolean(false);
        dataList.forEach(data -> {
            QueryWrapper query = dataService.query();
            query.eq(Data::getBatchNum, data.getBatchNum());
            long count = dataService.getMapper().selectCountByQuery(query);
            if (count > 0) {
                isTrue.set(true);
            }
        });

        if (isTrue.get()) {
            return CommonResult.fail("文件检测出重复批次号，请仔细检查后重新导入！");
        }
        dataService.saveBatch(dataList);
        //删除压缩文件
        FileUtils.deleteFile(destZip);
        return CommonResult.success("数据集上传成功！");
    }

    private boolean initFileInfo(String dest, Integer idMax, boolean directory, String fileName, List<Data> dataList, Data dataInfo) throws IOException {
        if (!directory) {
            String fileHeaderSuffix = StringUtils.substring(fileName, fileName.lastIndexOf("."), fileName.length());
            if (!fileHeaderSuffix.equals(".jpg") && !fileHeaderSuffix.equals(".jpeg") && !fileHeaderSuffix.equals(".png")) {
                return false;
            }
            File file = new File(dest, idMax + fileHeaderSuffix);
            File odlFile = new File(dest, fileName);
            log.info("更改用户上传文件名称:{}", odlFile.renameTo(file));
            String pathFileName = FileUploadUtils.getPathFileName(dest, idMax + fileHeaderSuffix);
            // 检查是否已标准抓住
            Data data = new Data();
            BeanUtils.copyProperties(dataInfo, data);
            if (checkLabeled(pathFileName)) {
                data.setLabeled(Boolean.TRUE);
            } else {
                data.setLabeled(Boolean.FALSE);
            }
            int lastSlashIndex = fileName.lastIndexOf('/');
            if (lastSlashIndex != -1) {
                fileName = fileName.substring(lastSlashIndex + 1);
            }
            data.setName(fileName);
            data.setUrl(pathFileName);
            dataList.add(data);
            return true;
        }
        return false;
    }

    // 检查有没有标注信息有的话把标注数据置为已标注
    private boolean checkLabeled(String pathFileName) {
        Path newPath = FileUtils.getTxtPath(pathFileName);

        // 检查替换后的文件是否存在
        return Files.exists(newPath);
    }

}
